/* 
   NSConcreteDictionary.h

   Copyright (C) 1995, 1996 Ovidiu Predescu and Mircea Oancea.
   All rights reserved.

   Author: Mircea Oancea <mircea@jupiter.elcom.pub.ro>

   This file is part of libFoundation.

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/

#ifndef __NSConcreteDictionary_h__
#define __NSConcreteDictionary_h__

#include <Foundation/NSUtilities.h>

/*
 * NSConcreteDictionary class
 */

@interface NSConcreteDictionary : NSDictionary
{
    NSMapTable*	table;
}

/* Allocating and Initializing an Dictionary */
- (id)initWithObjects:(id*)objects forKeys:(id*)keys 
  count:(unsigned int)count;
- (id)initWithDictionary:(NSDictionary*)dictionary;

/* Accessing keys and values */
- (id)objectForKey:(id)aKey;
- (unsigned int)count;
- (NSEnumerator *)keyEnumerator;

/* Private */
- (NSMapEnumerator)__keyEnumerator;

@end /* NSConcreteDictionary */

/*
 * NSConcreteMutableDictionary class
 */

@interface NSConcreteMutableDictionary : NSMutableDictionary
{
    NSMapTable	*table;
}

/* Allocating and Initializing an Dictionary */
- (id)init;
- (id)initWithCapacity:(unsigned int)aNumItems;


/* Modifying dictionary */
- (void)setObject:(id)anObject forKey:(id)aKey;
- (void)removeObjectForKey:(id)theKey;
- (void)removeAllObjects;

@end /* NSConcreteMutableDictionary */

/*
 * NSDictionary Enumerator classes
 */

@interface _NSDictionaryObjectEnumerator : NSObject
{
    NSDictionary* dict;
    NSEnumerator* keys;
}

- initWithDictionary:(NSDictionary*)_dict;
- nextObject;

@end /* _NSDictionaryObjectEnumerator */

/*
 * NSDictionary Enumerator classes
 */

@interface _NSConcreteDictionaryKeyEnumerator : NSObject
{
    NSDictionary*	dict;
    NSMapEnumerator	enumerator;
}

- initWithDictionary:(NSDictionary*)_dict;
- nextObject;

@end /* _NSConcreteDictionaryKeyEnumerator */

#endif /* __NSConcreteDictionary_h__ */
