/*
   NSFileHandle.m

   Copyright (C) 1995, 1996, 1997 Ovidiu Predescu and Mircea Oancea.
   All rights reserved.

   Author: Ovidiu Predescu <ovidiu@net-community.com>
   Date: May 1997

   This file is part of libFoundation.

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <Foundation/NSFileHandle.h>
#include "NSConcreteFileHandle.h"

@implementation NSFileHandle

+ (id)allocWithZone:(NSZone*)zone
{
    return (self == [NSFileHandle class])
	? [NSConcreteFileHandle allocWithZone:zone] 
	: NSAllocateObject(self, 0, zone);
}

+ (id)fileHandleForReadingAtPath:(NSString*)path
{
    return [[[self alloc]
		initWithPath:path flags:O_RDONLY createMode:0]
		autorelease];
}

+ (id)fileHandleForWritingAtPath:(NSString*)path
{
    return [[[self alloc]
		initWithPath:path flags:O_WRONLY createMode:0]
		autorelease];
}

+ (id)fileHandleForUpdatingAtPath:(NSString*)path
{
    return [[[self alloc]
		initWithPath:path flags:O_RDWR createMode:0]
		autorelease];
}

+ (id)fileHandleWithStandardInput
{
    return [[[self alloc] initWithFileDescriptor:0] autorelease];
}

+ (id)fileHandleWithStandardOutput
{
    return [[[self alloc] initWithFileDescriptor:1] autorelease];
}

+ (id)fileHandleWithStandardError
{
    return [[[self alloc] initWithFileDescriptor:2] autorelease];
}

+ (id)fileHandleWithNullDevice
{
    return [[NSNullDeviceFileHandle new] autorelease];
}

@end /* NSFileHandle */


NSString* NSFileHandleConnectionAcceptedNotification
    = @"NSFileHandleConnectionAcceptedNotification";

NSString* NSFileHandleReadCompletionNotification
    = @"NSFileHandleReadCompletionNotification";

NSString* NSFileHandleReadToEndOfFileCompletionNotification
    = @"NSFileHandleReadToEndOfFileCompletionNotification";

NSString* NSFileHandleNotificationFileHandleItem
    = @"NSFileHandleNotificationFileHandleItem";

NSString* NSFileHandleNotificationDataItem
    = @"NSFileHandleNotificationDataItem";

NSString* NSFileHandleNotificationMonitorModes
    = @"NSFileHandleNotificationMonitorModes";
