/* 
   NSInvocation.h

   Copyright (C) 1995, 1996 Ovidiu Predescu and Mircea Oancea.
   All rights reserved.

   Author: Ovidiu Predescu <ovidiu@bx.logicnet.ro>

   This file is part of libFoundation.

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/

#ifndef __NSInvocation_h__
#define __NSInvocation_h__

#include <Foundation/NSMethodSignature.h>

@interface NSInvocation : NSObject
{
    SEL selector;
    id target;
    NSMethodSignature* signature;
    arglist_t frame;
    void* returnValue;
    BOOL argumentsRetained;
    BOOL ownsFrame;
    BOOL ownsReturnValue;
    BOOL isValid;
    retval_t returnFrame;
}

/* Creating Invocations */
+ (NSInvocation*)invocationWithMethodSignature:(NSMethodSignature*)sig;

/* Managing Invocation Arguments */
- (BOOL)argumentsRetained;
- (void)getArgument:(void*)argumentLocation
	atIndex:(int)index;
- (void)getReturnValue:(void*)retLoc;
- (NSMethodSignature*)methodSignature;
- (void)retainArguments;
- (SEL)selector;
- (void)setArgument:(void*)argumentLocation
	atIndex:(int)index;
- (void)setReturnValue:(void*)retLoc;
- (void)setSelector:(SEL)selector;
- (void)setTarget:(id)target;
- (id)target;

/* Dispatching an Invocation */
- (void)invoke;
- (void)invokeWithTarget:(id)target;

/* Invalidate the receiver */
- (void)invalidate;

@end


@interface NSInvocation (Extensions)
- (void)setArgumentFrame:(void*)frame;
- (retval_t)returnFrame;
- (void*)returnValue;
@end /* NSInvocation (Extensions) */


#endif /* __NSInvocation_h__ */
