/* 
   NSPathUtilities.h

   Copyright (C) 1995, 1996 Ovidiu Predescu and Mircea Oancea.
   All rights reserved.

   Author: Mircea Oancea <mircea@jupiter.elcom.pub.ro>

   This file is part of libFoundation.

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/

#ifndef __NSPathUtilities_h__
#define __NSPathUtilities_h__

/*
 * User Informations
 */

extern NSString* NSUserName(void);
extern NSString* NSHomeDirectory(void);
extern NSString* NSHomeDirectoryForUser(NSString* userName);

extern NSString* NSFullUserName();
extern NSString* NSTemporaryDirectory();

/*
 * String file naming utilities
 */

@interface NSString(FilePathMethods)
+ (NSString*)pathWithComponents:(NSArray*)components;
- (NSArray*)pathComponents;
- (unsigned int)completePathIntoString:(NSString**)outputName
  caseSensitive:(BOOL)flag matchesIntoArray:(NSArray**)outputArray
  filterTypes:(NSArray*)filterTypes; 
- (const char *)fileSystemRepresentation;
- (BOOL)getFileSystemRepresentation:(char*)buffer
  maxLength:(unsigned int)maxLength;
- (BOOL)isAbsolutePath;
- (NSString*)lastPathComponent;
- (NSString*)pathExtension;
- (NSString*)stringByAbbreviatingWithTildeInPath;
- (NSString*)stringByAppendingPathComponent:(NSString*)aString;
- (NSString*)stringByAppendingPathExtension:(NSString*)aString;
- (NSString*)stringByDeletingLastPathComponent;
- (NSString*)stringByDeletingPathExtension;
- (NSString*)stringByExpandingTildeInPath;
- (NSString*)stringByResolvingSymlinksInPath;
- (NSString*)stringByStandardizingPath;
- (NSArray *)stringsByAppendingPaths:(NSArray *)paths;
@end

#endif /* __NSPathUtilities_h__ */
