/* 
   NSPipe.m

   Copyright (C) 1995, 1996, 1997 Ovidiu Predescu and Mircea Oancea.
   All rights reserved.

   Author: Ovidiu Predescu <ovidiu@net-community.com>

   This file is part of libFoundation.

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/

#include <errno.h>

#include <Foundation/common.h>
#include <Foundation/NSFileHandle.h>

@interface NSConcretePipe : NSPipe
{
    NSFileHandle* _readFd;
    NSFileHandle* _writeFd;
}
@end


@implementation NSConcretePipe
- (NSFileHandle*)fileHandleForReading		{ return _readFd; }
- (NSFileHandle*)fileHandleForWriting		{ return _writeFd; }

- (id)init
{
    int fildes[2];

    if (pipe (fildes) == -1) {
	NSLog (@"pipe system call failed: %s", strerror (errno));
	[self autorelease];
	return nil;
    }

    _readFd = [[NSFileHandle alloc] initWithFileDescriptor:fildes[0]
					closeOnDealloc:YES];
    _writeFd = [[NSFileHandle alloc] initWithFileDescriptor:fildes[1]
					closeOnDealloc:YES];
    return self;
}

- (void)dealloc
{
    [_readFd release];
    [_writeFd release];
    [super dealloc];
}

@end

@implementation NSPipe

+ pipe
{
    return [[[NSConcretePipe alloc] init] autorelease];
}

- (NSFileHandle*)fileHandleForReading
{
    [self shouldNotImplement:_cmd];
    return nil;
}

- (NSFileHandle*)fileHandleForWriting
{
    [self shouldNotImplement:_cmd];
    return nil;
}

- (id)init
{
    [self shouldNotImplement:_cmd];
    return nil;
}

@end
