/* 
   NSStringPropList.g

   Copyright (C) 1995, 1996 Ovidiu Predescu and Mircea Oancea.
   All rights reserved.

   Author: Ovidiu Predescu <ovidiu@bx.logicnet.ro>

   This file is part of libFoundation.

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
#header <<
#include <objc/objc.h>

#ifndef __Attrib_def__
#define __Attrib_def__
typedef id Attrib;
#endif

#define USER_ZZSYN
#define zzcr_attr NSStringPropertyListParser_zzcr_attr
void zzcr_attr(Attrib* attr, int token, char* text);
>>

#parser "NSStringPropertyListParser"

<<
static zzchar_t	*zzlextext;	/* text of most recently matched token */
static zzchar_t	*zzbegexpr;	/* beginning of last reg expr recogn. */
static zzchar_t	*zzendexpr;	/* beginning of last reg expr recogn. */
static int	zzbufsize;	/* number of characters in zzlextext */

#include "err.m"
#include "NSStringScanner.m"

#include <ctype.h>
#include <Foundation/NSString.h>
#include <Foundation/NSData.h>
#include <Foundation/NSArray.h>
#include <Foundation/NSDictionary.h>
#include <Foundation/NSAutoreleasePool.h>
#include <Foundation/NSUtilities.h>
#include <Foundation/NSException.h>
#include <Foundation/NSLock.h>
#include <Foundation/exceptions/GeneralExceptions.h>

static NSMutableString* NSStringPropertyListParser_errors;

extern NSLock* libFoundationLock;

id readPropertyListFrom(NSString* string)
{
    id plist = nil;
    id pool;

    [libFoundationLock lock];
    pool = [NSAutoreleasePool new];
    NSStringPropertyListParser_errors = [[NSMutableString new] autorelease];
    ANTLRs(plist = propertyList(), (char*)[string cString]);

    if(*[NSStringPropertyListParser_errors cString]) {
	[libFoundationLock unlock];
	THROW([[SyntaxErrorException new]
		setReason:NSStringPropertyListParser_errors]);
    }
    [plist retain];
    [pool release];
    [libFoundationLock unlock];

    return [[[plist autorelease] copy] autorelease];
}

id readDictionary(NSString *string)
{
    id plist = nil;
    id pool = [NSAutoreleasePool new];

    NSStringPropertyListParser_errors = [[NSMutableString new] autorelease];
    ANTLRs(plist = strings(), (char*)[string cString]);

    if(*[NSStringPropertyListParser_errors cString])
	THROW([[SyntaxErrorException new]
		setReason:NSStringPropertyListParser_errors]);
    plist = [plist copy];
    [pool release];

    return [plist autorelease];
}

static char make_hexa(char a, char b);

void zzcr_attr(Attrib* attr, int token, char* text)
{
    switch(token) {
	case DATA: {
	    int i, pos, len = strlen(text);
	    for(i = pos = 0; i < len; i += 2, pos++)
		text[pos] = make_hexa(text[i],
				      text[i + 1] ? text[i + 1] : '0');
	    *attr = [NSData dataWithBytes:text length:pos];
	    break;
	}
	case STRING:
	    *attr = [NSString stringWithCString:text];
	    break;
    }
}

void zzsyn(char* text, int tok, char* egroup,
    SetWordType* eset, int etok, int k, char*bad_text)
{
    [NSStringPropertyListParser_errors appendFormat:
			@"line %d: syntax error at \"%s\"",
			zzline, (tok == zzEOF_TOKEN) ? "EOF" : bad_text];
    if(!etok && !eset) {
	[NSStringPropertyListParser_errors appendString:@"\n"];
	return;
    }
    if (k==1)
	[NSStringPropertyListParser_errors appendString:@" missing"];
    else {
	[NSStringPropertyListParser_errors
		    appendFormat:@"; \"%s\" not", bad_text];
	if(zzset_deg(eset) > 1)
	    [NSStringPropertyListParser_errors appendString:@" in"];
    }
    if(zzset_deg(eset) > 0)
	zzedecode(eset);
    else
	[NSStringPropertyListParser_errors
		appendFormat:@" %s", zztokens[etok]];
    if(strlen(egroup) > 0)
	[NSStringPropertyListParser_errors appendFormat:@" in %s", egroup];
    [NSStringPropertyListParser_errors appendString:@"\n"];
}

static char make_hexa(char a, char b)
{
    char c = 0;

    if(isdigit(a))
	c = (a - '0') << 4;
    else if(isalpha(a))
	c = (tolower(a) - 'a' + 10) << 4;

    if(isdigit(b))
	c += b - '0';
    else if(isalpha(b))
	c += tolower(b) - 'a' + 10;

    return c;
}
>>

#lexaction <<
#include <Foundation/NSString.h>

static int level = 0;
extern NSMutableString* NSStringPropertyListParser_errors;

#define ERROR(msg) \
    [NSStringPropertyListParser_errors \
	appendFormat:[NSString stringWithCString:"line %d: " msg "\n"], zzline]
#define ERROR1(msg, arg) \
    [NSStringPropertyListParser_errors \
	appendFormat:[NSString stringWithCString:"line %d: " msg "\n"], zzline, arg]
>>

#lexclass COMMENT
#token "/\*"		<<level++; zzskip();>>
#token "~[\n]*\*/"	<<if(!--level) zzmode(START); zzskip();>>
#token "\n"		<<zzline++; zzskip();>>
#token "~[\n]+"		<<zzskip();>>

#lexclass LINE_COMMENT
#token "\n"		<<zzmode(START); zzline++; zzskip();>>
#token "~[\n]+"		<<zzskip();>>

#lexclass STRING_CLASS
#token STRING "\""	<<zzmode(START); zzreplstr("");>>
#token "\\"		<<zzmore();>>
#token "\\a"		<<{char a[2]={'\a',0}; zzreplstr(a);zzmore();}>>
#token "\\b"		<<{char a[2]={'\b',0}; zzreplstr(a);zzmore();}>>
#token "\\f"		<<{char a[2]={'\f',0}; zzreplstr(a);zzmore();}>>
#token "\\n"		<<{char a[2]={'\n',0}; zzreplstr(a);zzmore();}>>
#token "\\t"		<<{char a[2]={'\t',0}; zzreplstr(a);zzmore();}>>
#token "\\v"		<<{char a[2]={'\v',0}; zzreplstr(a);zzmore();}>>
#token "\\~[]"		<<{char a[2]={*zzendexpr,0}; zzreplstr(a);zzmore();}>>
#token "~[\"\\]+"	<<zzmore();>>

#lexclass DATA_CLASS
#token DATA "\>"	<<zzmode(START); zzreplstr("");>>
#token "[\ \t]+"	<<zzreplstr(""); zzmore();>>
#token "([0-9a-fA-F])+"	<<zzmore();>>
#token "~[]"		<<ERROR1("invalid character in description of NSData: '%c'", *zzendexpr); zzskip();>>

#lexclass START
#token "[\t\ ]+"	<<zzskip();>>
#token "\n"		<<zzline++; zzskip();>>
#token Eof "@"

#token "/\*"		<<zzmode(COMMENT); level++; zzskip();>>
#token "//"		<<zzmode(LINE_COMMENT); zzskip();>>
#token "\*/"		<<ERROR("unexpected comment terminator"); zzskip();>>
#token "\""		<<zzmode(STRING_CLASS); zzskip();>>
#token "\<"		<<zzmode(DATA_CLASS); zzskip();>>

#token STRING		"[a-zA-Z0-9_\@#/]([a-zA-Z0-9_\@#$./])*"


/*
 * Property List Parser
 */
propertyList > [id plist]:
<<$plist=nil;>>
	pobj Eof <<$plist=$1;>>
;
pobj:	  plist		<<$$=$1;>>
	| dict		<<$$=$1;>>
	| STRING	<<$$=$1;>>
	| DATA		<<$$=$1;>>
;
plist:
<<$$=[NSMutableArray array];>>
	"\(" {pobj <<[$$ addObject:$2];>> ({","} pobj <<[$$ addObject:$2];>>)*} "\)"
;
dict:
<<$$=[NSMutableDictionary dictionary];>>
	"\{" (pobj "=" pobj {";"} <<[$$ setObject:$3 forKey:$1];>>)* "\}"
;


/*
 * .string File Format
 */

strings > [id plist]:
<<$plist=[NSMutableDictionary dictionary];>>
	(STRING "=" STRING {";"} <<[$plist setObject:$3 forKey:$1];>>) *
;
