/* 
   NSThread.h

   Copyright (C) 1995, 1996 Ovidiu Predescu and Mircea Oancea.
   All rights reserved.

   Author: Mircea Oancea <mircea@jupiter.elcom.pub.ro>

   This file is part of libFoundation.

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/

#ifndef __NSThread_h__
#define __NSThread_h__

#include <Foundation/NSObject.h>

@class NSDate;
@class NSString;
@class NSMutableDictionary;
@class NSAutoreleasePool;
@class NSNotificationCenter;
@class NSNotificationQueue;
@class NSRunLoop;

enum {
    NSInteractiveThreadPriority,
    NSBackgroundThreadPriority,
    NSLowThreadPriority
};

@interface NSThread : NSObject
{
    /* global thread info */
    id threadDictionary;
    BOOL isRunning;
    id target;
    SEL selector;
    id arg;
    
    /* fast access to per-thread-global variables */
    id autoreleaseStack;
    void* exceptionStack;
    NSRunLoop* runLoop;
    struct _InstanceList* notificationQueues;
    NSNotificationQueue* defaultQueue;
    NSNotificationCenter* defaultCenter;
}

+ (NSThread*)currentThread;
+ (void)exit;
+ (BOOL)isMultiThreaded;
+ (void)sleepUntilDate:(NSDate*)aDate;
+ (void)detachNewThreadSelector:(SEL)aSelector 
  toTarget:(id)aTarget withObject:(id)anArgument;
- (NSMutableDictionary*)threadDictionary;

/* Private methods */

/* NSAutoreleasePool */
- (NSAutoreleasePool*)threadDefaultAutoreleasePool;
- (void)setThreadDefaultAutoreleasePool:(NSAutoreleasePool*)pool;

/* NSException */
- (void*)threadDefaultExceptionHandler;
- (void)setThreadDefaultExceptionHandler:(void*)handler;

/* NSRunLoop */
- (NSRunLoop*)threadRunLoop;
- (void)setThreadRunLoop:(NSRunLoop*)aLoop;

/* NSNotificationQueue */
- (struct _InstanceList*)threadNotificationQueues;
- (void)setThreadNotificationQueues:(struct _InstanceList*)theQueues;
- (NSNotificationQueue*)defaultNotificationQueue;
- (void)setDefaultNotificationQueue:(NSNotificationQueue*)aQueue;

/* NSNotificationCenter */
- (NSNotificationCenter*)defaultNotificationCenter;
- (void)setDefaultNotificationCenter:(NSNotificationCenter*)center;

@end /* NSThread */

/* Thead notifications */

extern NSString* NSWillBecomeMultiThreadedNotification;
extern NSString* NSThreadWillExitNotification;

#endif /* __NSThread_h__ */
