/* 
   NSUtilities.m

   Copyright (C) 1995, 1996 Ovidiu Predescu and Mircea Oancea.
   All rights reserved.

   Author: Ovidiu Predescu <ovidiu@bx.logicnet.ro>
	   Mircea Oancea <mircea@jupiter.elcom.pub.ro>

   This file is part of libFoundation.

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/

#include <Foundation/common.h>
#include <Foundation/NSString.h>
#include <Foundation/NSUtilities.h>
#include <Foundation/NSDate.h>
#include <Foundation/NSProcessInfo.h>
#include <Foundation/NSPosixFileDescriptor.h>
#include <Foundation/NSLock.h>

extern NSRecursiveLock* libFoundationLock;

/* 
 * Log a Message 
 */

void NSLog(NSString *format, ...)
{
    va_list ap;

    va_start(ap, format);
    NSLogv(format, ap);
    va_end(ap);
}

void NSLogv(NSString *format, va_list args)
{
    NSString* message = [NSString stringWithFormat:format arguments:args];
    NSCalendarDate* date = [NSCalendarDate calendarDate];
    id file = [NSPosixFileDescriptor descriptorWithStandardError];
    NSString* header;

    header = [NSString stringWithFormat:@"%@ %@ [%d]: ",
			[date descriptionWithCalendarFormat:@"%b %d %H:%M:%S"],
			[[NSProcessInfo processInfo] processName],
			getpid()];

    [libFoundationLock lock];
    [file writeString:header];
    [file writeString:message];
    [file writeString:@"\n"];
    [libFoundationLock unlock];
}

/* Support function for debugger */
NSString* _NSNewStringFromCString (const char* cString)
{
    return [NSString stringWithCString:cString];
}
